@@ -1,6 +1,4 @@
-from __future__ import print_function, division, absolute_import
-from fontTools.misc.py23 import *
-from fontTools.misc.textTools import safeEval, readHex
+from fontTools.misc.textTools import bytesjoin, safeEval, readHex
 from fontTools.misc.encodingTools import getEncoding
 from fontTools.ttLib import getSearchRange
 from fontTools.unicode import Unicode
@@ -8,7 +6,6 @@ from . import DefaultTable
 import sys
 import struct
 import array
-import operator
 import logging
 
 
@@ -17,21 +14,61 @@ log = logging.getLogger(__name__)
 
 def _make_map(font, chars, gids):
 	assert len(chars) == len(gids)
+	glyphNames = font.getGlyphNameMany(gids)
 	cmap = {}
-	glyphOrder = font.getGlyphOrder()
-	for char,gid in zip(chars,gids):
-		if gid is 0:
+	for char,gid,name in zip(chars,gids,glyphNames):
+		if gid == 0:
 			continue
-		try:
-			name = glyphOrder[gid]
-		except IndexError:
-			name = font.getGlyphName(gid)
 		cmap[char] = name
 	return cmap
 
 class table__c_m_a_p(DefaultTable.DefaultTable):
+	"""Character to Glyph Index Mapping Table
+
+	This class represents the `cmap <https://docs.microsoft.com/en-us/typography/opentype/spec/cmap>`_
+	table, which maps between input characters (in Unicode or other system encodings)
+	and glyphs within the font. The ``cmap`` table contains one or more subtables
+	which determine the mapping of of characters to glyphs across different platforms
+	and encoding systems.
+
+	``table__c_m_a_p`` objects expose an accessor ``.tables`` which provides access
+	to the subtables, although it is normally easier to retrieve individual subtables
+	through the utility methods described below. To add new subtables to a font,
+	first determine the subtable format (if in doubt use format 4 for glyphs within
+	the BMP, format 12 for glyphs outside the BMP, and format 14 for Unicode Variation
+	Sequences) construct subtable objects with ``CmapSubtable.newSubtable(format)``,
+	and append them to the ``.tables`` list.
+
+	Within a subtable, the mapping of characters to glyphs is provided by the ``.cmap``
+	attribute.
+
+	Example::
+
+		cmap4_0_3 = CmapSubtable.newSubtable(4)
+		cmap4_0_3.platformID = 0
+		cmap4_0_3.platEncID = 3
+		cmap4_0_3.language = 0
+		cmap4_0_3.cmap = { 0xC1: "Aacute" }
+
+		cmap = newTable("cmap")
+		cmap.tableVersion = 0
+		cmap.tables = [cmap4_0_3]
+	"""
 
 	def getcmap(self, platformID, platEncID):
+		"""Returns the first subtable which matches the given platform and encoding.
+
+		Args:
+			platformID (int): The platform ID. Use 0 for Unicode, 1 for Macintosh
+				(deprecated for new fonts), 2 for ISO (deprecated) and 3 for Windows.
+			encodingID (int): Encoding ID. Interpretation depends on the platform ID.
+				See the OpenType specification for details.
+
+		Returns:
+			An object which is a subclass of :py:class:`CmapSubtable` if a matching
+			subtable is found within the font, or ``None`` otherwise.
+		"""
+
 		for subtable in self.tables:
 			if (subtable.platformID == platformID and
 					subtable.platEncID == platEncID):
@@ -39,13 +76,27 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 		return None # not found
 
 	def getBestCmap(self, cmapPreferences=((3, 10), (0, 6), (0, 4), (3, 1), (0, 3), (0, 2), (0, 1), (0, 0))):
-		"""Return the 'best' unicode cmap dictionary available in the font,
-		or None, if no unicode cmap subtable is available.
+		"""Returns the 'best' Unicode cmap dictionary available in the font
+		or ``None``, if no Unicode cmap subtable is available.
 
 		By default it will search for the following (platformID, platEncID)
-		pairs:
-			(3, 10), (0, 6), (0, 4), (3, 1), (0, 3), (0, 2), (0, 1), (0, 0)
-		This can be customized via the cmapPreferences argument.
+		pairs in order::
+
+				(3, 10), # Windows Unicode full repertoire
+				(0, 6),  # Unicode full repertoire (format 13 subtable)
+				(0, 4),  # Unicode 2.0 full repertoire
+				(3, 1),  # Windows Unicode BMP
+				(0, 3),  # Unicode 2.0 BMP
+				(0, 2),  # Unicode ISO/IEC 10646
+				(0, 1),  # Unicode 1.1
+				(0, 0)   # Unicode 1.0
+
+		This particular order matches what HarfBuzz uses to choose what
+		subtable to use by default. This order prefers the largest-repertoire
+		subtable, and among those, prefers the Windows-platform over the
+		Unicode-platform as the former has wider support.
+
+		This order can be customized via the ``cmapPreferences`` argument.
 		"""
 		for platformID, platEncID in cmapPreferences:
 			cmapSubtable = self.getcmap(platformID, platEncID)
@@ -54,12 +105,20 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 		return None  # None of the requested cmap subtables were found
 
 	def buildReversed(self):
-		"""Returns a reverse cmap such as {'one':{0x31}, 'A':{0x41,0x391}}.
+		"""Builds a reverse mapping dictionary
+
+		Iterates over all Unicode cmap tables and returns a dictionary mapping
+		glyphs to sets of codepoints, such as::
+
+			{
+				'one': {0x31}
+				'A': {0x41,0x391}
+			}
 
 		The values are sets of Unicode codepoints because
 		some fonts map different codepoints to the same glyph.
-		For example, U+0041 LATIN CAPITAL LETTER A and U+0391
-		GREEK CAPITAL LETTER ALPHA are sometimes the same glyph.
+		For example, ``U+0041 LATIN CAPITAL LETTER A`` and ``U+0391
+		GREEK CAPITAL LETTER ALPHA`` are sometimes the same glyph.
 		"""
 		result = {}
 		for subtable in self.tables:
@@ -102,9 +161,15 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 			else:
 				seenOffsets[offset] = i
 			tables.append(table)
+		if ttFont.lazy is False:  # Be lazy for None and True
+			self.ensureDecompiled()
+
+	def ensureDecompiled(self):
+		for st in self.tables:
+			st.ensureDecompiled()
 
 	def compile(self, ttFont):
-		self.tables.sort()    # sort according to the spec; see CmapSubtable.__lt__()
+		self.tables.sort()  # sort according to the spec; see CmapSubtable.__lt__()
 		numSubTables = len(self.tables)
 		totalOffset = 4 + 8 * numSubTables
 		data = struct.pack(">HH", self.tableVersion, numSubTables)
@@ -112,13 +177,11 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 		seen = {}  # Some tables are the same object reference. Don't compile them twice.
 		done = {}  # Some tables are different objects, but compile to the same data chunk
 		for table in self.tables:
-			try:
-				offset = seen[id(table.cmap)]
-			except KeyError:
+			offset = seen.get(id(table.cmap))
+			if offset is None:
 				chunk = table.compile(ttFont)
-				if chunk in done:
-					offset = done[chunk]
-				else:
+				offset = done.get(chunk)
+				if offset is None:
 					offset = seen[id(table.cmap)] = done[chunk] = totalOffset + len(tableData)
 					tableData = tableData + chunk
 			data = data + struct.pack(">HHl", table.platformID, table.platEncID, offset)
@@ -147,6 +210,16 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 
 
 class CmapSubtable(object):
+	"""Base class for all cmap subtable formats.
+
+	Subclasses which handle the individual subtable formats are named
+	``cmap_format_0``, ``cmap_format_2`` etc. Use :py:meth:`getSubtableClass`
+	to retrieve the concrete subclass, or :py:meth:`newSubtable` to get a
+	new subtable object for a given format.
+
+	The object exposes a ``.cmap`` attribute, which contains a dictionary mapping
+	character codepoints to glyph names.
+	"""
 
 	@staticmethod
 	def getSubtableClass(format):
@@ -155,7 +228,8 @@ class CmapSubtable(object):
 
 	@staticmethod
 	def newSubtable(format):
-		"""Return a new instance of a subtable for format."""
+		"""Return a new instance of a subtable for the given format
+		."""
 		subtableClass = CmapSubtable.getSubtableClass(format)
 		return subtableClass(format)
 
@@ -163,6 +237,17 @@ class CmapSubtable(object):
 		self.format = format
 		self.data = None
 		self.ttFont = None
+		self.platformID = None  #: The platform ID of this subtable
+		self.platEncID = None   #: The encoding ID of this subtable (interpretation depends on ``platformID``)
+		self.language = None    #: The language ID of this subtable (Macintosh platform only)
+
+	def ensureDecompiled(self):
+		if self.data is None:
+			return
+		self.decompile(None, None) # use saved data.
+		self.data = None	# Once this table has been decompiled, make sure we don't
+							# just return the original data. Also avoids recursion when
+							# called with an attribute that the cmap subtable doesn't have.
 
 	def __getattr__(self, attr):
 		# allow lazy decompilation of subtables.
@@ -170,10 +255,7 @@ class CmapSubtable(object):
 			raise AttributeError(attr)
 		if self.data is None:
 			raise AttributeError(attr)
-		self.decompile(None, None) # use saved data.
-		self.data = None	# Once this table has been decompiled, make sure we don't
-							# just return the original data. Also avoids recursion when
-							# called with an attribute that the cmap subtable doesn't have.
+		self.ensureDecompiled()
 		return getattr(self, attr)
 
 	def decompileHeader(self, data, ttFont):
@@ -200,20 +282,22 @@ class CmapSubtable(object):
 	def getEncoding(self, default=None):
 		"""Returns the Python encoding name for this cmap subtable based on its platformID,
 		platEncID, and language.  If encoding for these values is not known, by default
-		None is returned.  That can be overriden by passing a value to the default
+		``None`` is returned.  That can be overridden by passing a value to the ``default``
 		argument.
 
 		Note that if you want to choose a "preferred" cmap subtable, most of the time
-		self.isUnicode() is what you want as that one only returns true for the modern,
+		``self.isUnicode()`` is what you want as that one only returns true for the modern,
 		commonly used, Unicode-compatible triplets, not the legacy ones.
 		"""
 		return getEncoding(self.platformID, self.platEncID, self.language, default)
 
 	def isUnicode(self):
+		"""Returns true if the characters are interpreted as Unicode codepoints."""
 		return (self.platformID == 0 or
 			(self.platformID == 3 and self.platEncID in [0, 1, 10]))
 
 	def isSymbol(self):
+		"""Returns true if the subtable is for the Symbol encoding (3,0)"""
 		return self.platformID == 3 and self.platEncID == 0
 
 	def _writeCodes(self, codes, writer):
@@ -246,7 +330,7 @@ class cmap_format_0(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
@@ -254,7 +338,7 @@ class cmap_format_0(CmapSubtable):
 		data = self.data # decompileHeader assigns the data after the header to self.data
 		assert 262 == self.length, "Format 0 cmap subtable not 262 bytes"
 		gids = array.array("B")
-		gids.fromstring(self.data)
+		gids.frombytes(self.data)
 		charCodes = list(range(len(gids)))
 		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
@@ -268,7 +352,7 @@ class cmap_format_0(CmapSubtable):
 		valueList = [getGlyphID(cmap[i]) if i in cmap else 0 for i in range(256)]
 
 		gids = array.array("B", valueList)
-		data = struct.pack(">HHH", 0, 262, self.language) + gids.tostring()
+		data = struct.pack(">HHH", 0, 262, self.language) + gids.tobytes()
 		assert len(data) == 262
 		return data
 
@@ -310,15 +394,15 @@ class cmap_format_2(CmapSubtable):
 		# so that we are more likely to be able to combine glypharray GID subranges.
 		# This means that we have a problem when minGI is > 32K
 		# Since the final gi is reconstructed from the glyphArray GID by:
-		#    (short)finalGID = (gid +  idDelta) % 0x10000),
+		#    (short)finalGID = (gid + idDelta) % 0x10000),
 		# we can get from a glypharray GID of 1 to a final GID of 65K by subtracting 2, and casting the
 		# negative number to an unsigned short.
 
-		if  (minGI > 1):
-			if  minGI > 0x7FFF:
+		if (minGI > 1):
+			if minGI > 0x7FFF:
 				subHeader.idDelta = -(0x10000 - minGI) -1
 			else:
-				subHeader.idDelta =  minGI -1
+				subHeader.idDelta = minGI -1
 			idDelta = subHeader.idDelta
 			for i in range(subHeader.entryCount):
 				gid = subHeader.glyphIndexArray[i]
@@ -327,7 +411,7 @@ class cmap_format_2(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
@@ -338,7 +422,7 @@ class cmap_format_2(CmapSubtable):
 		maxSubHeaderindex = 0
 		# get the key array, and determine the number of subHeaders.
 		allKeys = array.array("H")
-		allKeys.fromstring(data[:512])
+		allKeys.frombytes(data[:512])
 		data = data[512:]
 		if sys.byteorder != "big": allKeys.byteswap()
 		subHeaderKeys = [ key//8 for key in allKeys]
@@ -354,13 +438,13 @@ class cmap_format_2(CmapSubtable):
 			pos += 8
 			giDataPos = pos + subHeader.idRangeOffset-2
 			giList = array.array("H")
-			giList.fromstring(data[giDataPos:giDataPos + subHeader.entryCount*2])
+			giList.frombytes(data[giDataPos:giDataPos + subHeader.entryCount*2])
 			if sys.byteorder != "big": giList.byteswap()
 			subHeader.glyphIndexArray = giList
 			subHeaderList.append(subHeader)
 		# How this gets processed.
 		# Charcodes may be one or two bytes.
-		# The first byte of a charcode is mapped through the  subHeaderKeys, to select
+		# The first byte of a charcode is mapped through the subHeaderKeys, to select
 		# a subHeader. For any subheader but 0, the next byte is then mapped through the
 		# selected subheader. If subheader Index 0 is selected, then the byte itself is
 		# mapped through the subheader, and there is no second byte.
@@ -442,13 +526,12 @@ class cmap_format_2(CmapSubtable):
 		charCodes = [item[0] for item in items]
 		names = [item[1] for item in items]
 		nameMap = ttFont.getReverseGlyphMap()
-		lenCharCodes = len(charCodes)
 		try:
-			gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+			gids = [nameMap[name] for name in names]
 		except KeyError:
 			nameMap = ttFont.getReverseGlyphMap(rebuild=True)
 			try:
-				gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+				gids = [nameMap[name] for name in names]
 			except KeyError:
 				# allow virtual GIDs in format 2 tables
 				gids = []
@@ -458,7 +541,7 @@ class cmap_format_2(CmapSubtable):
 					except KeyError:
 						try:
 							if (name[:3] == 'gid'):
-								gid = eval(name[3:])
+								gid = int(name[3:])
 							else:
 								gid = ttFont.getGlyphID(name)
 						except:
@@ -466,17 +549,17 @@ class cmap_format_2(CmapSubtable):
 
 					gids.append(gid)
 
-		# Process the (char code to gid) item list  in char code order.
+		# Process the (char code to gid) item list in char code order.
 		# By definition, all one byte char codes map to subheader 0.
 		# For all the two byte char codes, we assume that the first byte maps maps to the empty subhead (with an entry count of 0,
 		# which defines all char codes in its range to map to notdef) unless proven otherwise.
 		# Note that since the char code items are processed in char code order, all the char codes with the
 		# same first byte are in sequential order.
 
-		subHeaderKeys = [ kEmptyTwoCharCodeRange for x in  range(256)] # list of indices into subHeaderList.
+		subHeaderKeys = [kEmptyTwoCharCodeRange for x in range(256)] # list of indices into subHeaderList.
 		subHeaderList = []
 
-		# We force this subheader entry 0  to exist in the subHeaderList in the case where some one comes up
+		# We force this subheader entry 0 to exist in the subHeaderList in the case where some one comes up
 		# with a cmap where all the one byte char codes map to notdef,
 		# with the result that the subhead 0 would not get created just by processing the item list.
 		charCode = charCodes[0]
@@ -549,7 +632,7 @@ class cmap_format_2(CmapSubtable):
 		for index in range(subheadRangeLen):
 			subHeader = subHeaderList[index]
 			subHeader.idRangeOffset = 0
-			for j  in range(index):
+			for j in range(index):
 				prevSubhead = subHeaderList[j]
 				if prevSubhead.glyphIndexArray == subHeader.glyphIndexArray: # use the glyphIndexArray subarray
 					subHeader.idRangeOffset = prevSubhead.idRangeOffset - (index-j)*8
@@ -684,7 +767,7 @@ class cmap_format_4(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
@@ -697,7 +780,7 @@ class cmap_format_4(CmapSubtable):
 		segCount = segCountX2 // 2
 
 		allCodes = array.array("H")
-		allCodes.fromstring(data)
+		allCodes.frombytes(data)
 		self.data = data = None
 
 		if sys.byteorder != "big": allCodes.byteswap()
@@ -720,7 +803,6 @@ class cmap_format_4(CmapSubtable):
 			start = startCode[i]
 			delta = idDelta[i]
 			rangeOffset = idRangeOffset[i]
-			# *someone* needs to get killed.
 			partial = rangeOffset // 2 - start + i - len(idRangeOffset)
 
 			rangeCharCodes = list(range(startCode[i], endCode[i] + 1))
@@ -730,7 +812,7 @@ class cmap_format_4(CmapSubtable):
 			else:
 				for charCode in rangeCharCodes:
 					index = charCode + partial
-					assert (index < lenGIArray), "In format 4 cmap, range (%d), the calculated index (%d) into the glyph index array  is not less than the length of the array (%d) !" % (i, index, lenGIArray)
+					assert (index < lenGIArray), "In format 4 cmap, range (%d), the calculated index (%d) into the glyph index array is not less than the length of the array (%d) !" % (i, index, lenGIArray)
 					if glyphIndexArray[index] != 0:  # if not missing glyph
 						glyphID = glyphIndexArray[index] + delta
 					else:
@@ -744,20 +826,19 @@ class cmap_format_4(CmapSubtable):
 			return struct.pack(">HHH", self.format, self.length, self.language) + self.data
 
 		charCodes = list(self.cmap.keys())
-		lenCharCodes = len(charCodes)
-		if lenCharCodes == 0:
+		if not charCodes:
 			startCode = [0xffff]
 			endCode = [0xffff]
 		else:
 			charCodes.sort()
-			names = list(map(operator.getitem, [self.cmap]*lenCharCodes, charCodes))
+			names = [self.cmap[code] for code in charCodes]
 			nameMap = ttFont.getReverseGlyphMap()
 			try:
-				gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+				gids = [nameMap[name] for name in names]
 			except KeyError:
 				nameMap = ttFont.getReverseGlyphMap(rebuild=True)
 				try:
-					gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+					gids = [nameMap[name] for name in names]
 				except KeyError:
 					# allow virtual GIDs in format 4 tables
 					gids = []
@@ -767,7 +848,7 @@ class cmap_format_4(CmapSubtable):
 						except KeyError:
 							try:
 								if (name[:3] == 'gid'):
-									gid = eval(name[3:])
+									gid = int(name[3:])
 								else:
 									gid = ttFont.getGlyphID(name)
 							except:
@@ -775,7 +856,8 @@ class cmap_format_4(CmapSubtable):
 
 						gids.append(gid)
 			cmap = {}  # code:glyphID mapping
-			list(map(operator.setitem, [cmap]*len(charCodes), charCodes, gids))
+			for code, gid in zip(charCodes, gids):
+				cmap[code] = gid
 
 			# Build startCode and endCode lists.
 			# Split the char codes in ranges of consecutive char codes, then split
@@ -807,11 +889,10 @@ class cmap_format_4(CmapSubtable):
 			indices = []
 			for charCode in range(startCode[i], endCode[i] + 1):
 				indices.append(cmap[charCode])
-			if  (indices == list(range(indices[0], indices[0] + len(indices)))):
+			if (indices == list(range(indices[0], indices[0] + len(indices)))):
 				idDelta.append((indices[0] - startCode[i]) % 0x10000)
 				idRangeOffset.append(0)
 			else:
-				# someone *definitely* needs to get killed.
 				idDelta.append(0)
 				idRangeOffset.append(2 * (len(endCode) + len(glyphIndexArray) - i))
 				glyphIndexArray.extend(indices)
@@ -829,7 +910,7 @@ class cmap_format_4(CmapSubtable):
 		if sys.byteorder != "big": charCodeArray.byteswap()
 		if sys.byteorder != "big": idDeltaArray.byteswap()
 		if sys.byteorder != "big": restArray.byteswap()
-		data = charCodeArray.tostring() + idDeltaArray.tostring() + restArray.tostring()
+		data = charCodeArray.tobytes() + idDeltaArray.tobytes() + restArray.tobytes()
 
 		length = struct.calcsize(cmap_format_4_format) + len(data)
 		header = struct.pack(cmap_format_4_format, self.format, length, self.language,
@@ -855,7 +936,7 @@ class cmap_format_6(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
@@ -867,7 +948,7 @@ class cmap_format_6(CmapSubtable):
 		data = data[4:]
 		#assert len(data) == 2 * entryCount  # XXX not true in Apple's Helvetica!!!
 		gids = array.array("H")
-		gids.fromstring(data[:2 * int(entryCount)])
+		gids.frombytes(data[:2 * int(entryCount)])
 		if sys.byteorder != "big": gids.byteswap()
 		self.data = data = None
 
@@ -888,7 +969,7 @@ class cmap_format_6(CmapSubtable):
 			]
 			gids = array.array("H", valueList)
 			if sys.byteorder != "big": gids.byteswap()
-			data = gids.tostring()
+			data = gids.tobytes()
 		else:
 			data = b""
 			firstCode = 0
@@ -932,7 +1013,7 @@ class cmap_format_12_or_13(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
@@ -955,15 +1036,14 @@ class cmap_format_12_or_13(CmapSubtable):
 		if self.data:
 			return struct.pack(">HHLLL", self.format, self.reserved, self.length, self.language, self.nGroups) + self.data
 		charCodes = list(self.cmap.keys())
-		lenCharCodes = len(charCodes)
 		names = list(self.cmap.values())
 		nameMap = ttFont.getReverseGlyphMap()
 		try:
-			gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+			gids = [nameMap[name] for name in names]
 		except KeyError:
 			nameMap = ttFont.getReverseGlyphMap(rebuild=True)
 			try:
-				gids = list(map(operator.getitem, [nameMap]*lenCharCodes, names))
+				gids = [nameMap[name] for name in names]
 			except KeyError:
 				# allow virtual GIDs in format 12 tables
 				gids = []
@@ -973,7 +1053,7 @@ class cmap_format_12_or_13(CmapSubtable):
 					except KeyError:
 						try:
 							if (name[:3] == 'gid'):
-								gid = eval(name[3:])
+								gid = int(name[3:])
 							else:
 								gid = ttFont.getGlyphID(name)
 						except:
@@ -982,7 +1062,8 @@ class cmap_format_12_or_13(CmapSubtable):
 					gids.append(gid)
 
 		cmap = {}  # code:glyphID mapping
-		list(map(operator.setitem, [cmap]*len(charCodes), charCodes, gids))
+		for code, gid in zip(charCodes, gids):
+			cmap[code] = gid
 
 		charCodes.sort()
 		index = 0
@@ -991,7 +1072,7 @@ class cmap_format_12_or_13(CmapSubtable):
 		lastGlyphID = startGlyphID - self._format_step
 		lastCharCode = startCharCode - 1
 		nGroups = 0
-		dataList =  []
+		dataList = []
 		maxIndex = len(charCodes)
 		for index in range(maxIndex):
 			charCode = charCodes[index]
@@ -1073,12 +1154,12 @@ class cmap_format_13(cmap_format_12_or_13):
 		return (glyphID == lastGlyphID) and (charCode == 1 + lastCharCode)
 
 
-def  cvtToUVS(threeByteString):
+def cvtToUVS(threeByteString):
 	data = b"\0" + threeByteString
 	val, = struct.unpack(">L", data)
 	return val
 
-def  cvtFromUVS(val):
+def cvtFromUVS(val):
 	assert 0 <= val < 0x1000000
 	fourByteString = struct.pack(">L", val)
 	return fourByteString[1:]
@@ -1105,7 +1186,7 @@ class cmap_format_14(CmapSubtable):
 		uvsDict = {}
 		recOffset = 0
 		for n in range(self.numVarSelectorRecords):
-			uvs, defOVSOffset, nonDefUVSOffset =  struct.unpack(">3sLL", data[recOffset:recOffset +11])
+			uvs, defOVSOffset, nonDefUVSOffset = struct.unpack(">3sLL", data[recOffset:recOffset +11])
 			recOffset += 11
 			varUVS = cvtToUVS(uvs)
 			if defOVSOffset:
@@ -1135,7 +1216,7 @@ class cmap_format_14(CmapSubtable):
 					startOffset += 5
 					uv = cvtToUVS(uv)
 					glyphName = self.ttFont.getGlyphName(gid)
-					localUVList.append( [uv, glyphName] )
+					localUVList.append((uv, glyphName))
 				try:
 					uvsDict[varUVS].extend(localUVList)
 				except KeyError:
@@ -1147,9 +1228,6 @@ class cmap_format_14(CmapSubtable):
 		writer.begintag(self.__class__.__name__, [
 				("platformID", self.platformID),
 				("platEncID", self.platEncID),
-				("format", self.format),
-				("length", self.length),
-				("numVarSelectorRecords", self.numVarSelectorRecords),
 				])
 		writer.newline()
 		uvsDict = self.uvsDict
@@ -1158,25 +1236,27 @@ class cmap_format_14(CmapSubtable):
 			uvList = uvsDict[uvs]
 			uvList.sort(key=lambda item: (item[1] is not None, item[0], item[1]))
 			for uv, gname in uvList:
-				if gname is None:
-					gname = "None"
-				# I use the arg rather than th keyword syntax in order to preserve the attribute order.
-				writer.simpletag("map", [ ("uvs",hex(uvs)), ("uv",hex(uv)), ("name", gname)]  )
+				attrs = [("uv", hex(uv)), ("uvs", hex(uvs))]
+				if gname is not None:
+					attrs.append(("name", gname))
+				writer.simpletag("map", attrs)
 				writer.newline()
 		writer.endtag(self.__class__.__name__)
 		writer.newline()
 
 	def fromXML(self, name, attrs, content, ttFont):
-		self.format = safeEval(attrs["format"])
-		self.length = safeEval(attrs["length"])
-		self.numVarSelectorRecords = safeEval(attrs["numVarSelectorRecords"])
-		self.language = 0xFF # provide a value so that  CmapSubtable.__lt__() won't fail
+		self.language = 0xFF # provide a value so that CmapSubtable.__lt__() won't fail
 		if not hasattr(self, "cmap"):
 			self.cmap = {} # so that clients that expect this to exist in a cmap table won't fail.
 		if not hasattr(self, "uvsDict"):
 			self.uvsDict = {}
 			uvsDict = self.uvsDict
 
+		# For backwards compatibility reasons we accept "None" as an indicator
+		# for "default mapping", unless the font actually has a glyph named
+		# "None".
+		_hasGlyphNamedNone = None
+
 		for element in content:
 			if not isinstance(element, tuple):
 				continue
@@ -1185,13 +1265,16 @@ class cmap_format_14(CmapSubtable):
 				continue
 			uvs = safeEval(attrs["uvs"])
 			uv = safeEval(attrs["uv"])
-			gname = attrs["name"]
+			gname = attrs.get("name")
 			if gname == "None":
-				gname = None
+				if _hasGlyphNamedNone is None:
+					_hasGlyphNamedNone = "None" in ttFont.getGlyphOrder()
+				if not _hasGlyphNamedNone:
+					gname = None
 			try:
-				uvsDict[uvs].append( [uv, gname])
+				uvsDict[uvs].append((uv, gname))
 			except KeyError:
-				uvsDict[uvs] = [ [uv, gname] ]
+				uvsDict[uvs] = [(uv, gname)]
 
 	def compile(self, ttFont):
 		if self.data:
@@ -1281,7 +1364,7 @@ class cmap_format_unknown(CmapSubtable):
 
 	def decompile(self, data, ttFont):
 		# we usually get here indirectly from the subtable __getattr__ function, in which case both args must be None.
-		# If not, someone is calling  the subtable decompile() directly, and must provide both args.
+		# If not, someone is calling the subtable decompile() directly, and must provide both args.
 		if data is not None and ttFont is not None:
 			self.decompileHeader(data, ttFont)
 		else:
